/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#statusContainer {
  position: absolute;
  bottom: 0px;
  z-index: 100;
  background-color: grey;
  color: white;
  margin: 0px;
  padding: 0px;
}

#statusContainer li {
  width: 100vw;
  max-height: 25vh;
  overflow-y: auto;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-image-dropdown {
  overflow-y: auto;
  display: flex;
  flex-direction: column;
}

.neuroglancer-image-dropdown .neuroglancer-shader-code-widget {
  height: 8em;
  border: 1px solid transparent;
}

.neuroglancer-image-dropdown-top-row {
  display: flex;
  flex-direction: row;
  align-items: center;
}

.neuroglancer-image-layer-shader-overlay .neuroglancer-shader-code-widget {
  width: 80vw;
  height: 80vh;
}

.neuroglancer-selection-details-value-grid {
  display: grid;
  grid-auto-rows: auto;
  font-family: monospace;
  font-size: medium;
  align-items: center;
}

.neuroglancer-selection-details-value-grid-dim {
  grid-column: dim;
  color: #ff6;
}

.neuroglancer-selection-details-value-grid-dim::after {
  content: "=";
  color: #aaa;
}

.neuroglancer-selection-details-value-grid-coord {
  grid-column: coord;
  color: #aaa;
  margin-right: 1ch;
}

.neuroglancer-selection-details-value-grid-value {
  grid-column: value;
  user-select: text;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-layer-data-sources-tab {
  display: flex;
  flex-direction: column;
}

.neuroglancer-layer-data-sources-container {
  overflow-y: auto;
  display: flex;
  flex-direction: column;
  flex: 1;
  height: 0px;
}

.neuroglancer-layer-data-source-url-input input.autocomplete-input {
  color: #fff;
}

.neuroglancer-layer-data-source-url-input input.autocomplete-hint {
  color: #aaa;
}

.neuroglancer-layer-data-source-url-input .autocomplete-dropdown {
  background-color: #333;
}

.neuroglancer-layer-data-sources-source-id {
  font-weight: bold;
}

.neuroglancer-layer-data-sources-source-id:not(:empty)::before {
  content: "[";
}

.neuroglancer-layer-data-sources-source-id:not(:empty)::after {
  content: "]";
}

.neuroglancer-layer-data-sources-source-id:not(:empty) {
  margin-right: 4px;
  color: #0ff;
}

.neuroglancer-layer-data-sources-source-type {
}

.neuroglancer-layer-data-sources-source-messages:empty {
  display: none;
}

.neuroglancer-layer-data-sources-source-messages {
  margin-top: 0px;
  list-style: none;
  padding-left: 0px;
}

li.neuroglancer-message {
  word-wrap: break-word;
  user-select: text;
}

li.neuroglancer-message-error {
  color: red;
}

li.neuroglancer-message-warning {
  color: yellow;
}

li.neuroglancer-message-info {
  color: #ccc;
}


.neuroglancer-layer-data-sources-source-default {
  display: block;
  margin-bottom: 4px;
}

.neuroglancer-layer-data-sources-info-line[data-is-active="false"] {
  text-decoration: solid line-through #caa;
}

.neuroglancer-layer-data-sources-tab-type-detection {
  position: sticky;
  bottom: 0px;
  background-color: #ff9;
  color: black;
  text-align: center;
  cursor: pointer;
}

.neuroglancer-layer-data-sources-tab-type-detection:hover {
  background-color: #ffc;
}

.neuroglancer-layer-data-sources-tab-type-detection-type {
  font-weight: bold;
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
.neuroglancer-icon {
  display: inline-flex;
  text-decoration: none;
  align-self: center;
  white-space: nowrap;
  padding-left: 2px;
  padding-right: 2px;
  min-width: 18px;
  min-height: 18px;
  border-radius: 20%;
  align-items: center;
  justify-content: center;
  color: white;
  font: 12px sans-serif;
  cursor: pointer;
  font-weight: 900;
}

.neuroglancer-icon svg {
  width: 16px;
  height: 16px;
  fill: transparent;
  stroke: white;
  stroke-width: 2;
  stroke-linecap: round;
  stroke-linejoin: round;
}

.neuroglancer-icon:hover {
  background-color: #db4437;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-coordinate-space-transform-widget {
  display: grid;
  justify-items: stretch;
}

.neuroglancer-coordinate-space-transform-widget input {
  font-family: monospace;
  justify-self: stretch;
  background-color: black;
  color: white;
  padding: 2px;
}

.neuroglancer-coordinate-space-transform-widget *[data-will-be-deleted="true"] {
  color: #666;
}

.neuroglancer-coordinate-space-transform-label {
  font-style: italic;
  color: #fcc;
  align-self: center;
}

.neuroglancer-coordinate-space-transform-input-lower-label {
  grid-row: sourceLower;
  grid-column: outputLabel / sourceDim 1;
}

.neuroglancer-coordinate-space-transform-translation-label {
  grid-row: sourceLabel / outputDim 1;
  grid-column: sourceDim -1;
  writing-mode: vertical-lr;
  text-orientation: mixed;
  align-self: end;
  justify-self: center;
  margin-bottom: 2px;
}

.neuroglancer-coordinate-space-transform-input-lower-label, .neuroglancer-coordinate-space-transform-input-upper-label , .neuroglancer-coordinate-space-transform-input-scale-label {
  text-align: right;
  margin-right: 4px;
}

.neuroglancer-coordinate-space-transform-input-upper-label {
  grid-row: sourceUpper;
  grid-column: headerStart / headerEnd;
}

.neuroglancer-coordinate-space-transform-input-scale-label {
  grid-row: 5;
  grid-column: headerStart / headerEnd;
}

.neuroglancer-coordinate-space-transform-source-label {
  grid-row: sourceLabel;
  grid-column: sourceDim 1 / sourceDim -1;
  text-align: center;
}

.neuroglancer-coordinate-space-transform-output-label {
  grid-row: outputDim 1 / outputDim -1;
  grid-column: outputLabel;
  writing-mode: vertical-lr;
  text-orientation: mixed;
  align-self: center;
  justify-self: end;
  margin-right: 2px;
}

.neuroglancer-coordinate-space-transform-input-name {
  text-align: center;
}

.neuroglancer-coordinate-space-transform-input-name, input.neuroglancer-coordinate-space-transform-output-name {
  font-family: monospace;
  color: #ff6;
}

.neuroglancer-coordinate-space-transform-scale {
  text-align: right;
}

.neuroglancer-coordinate-space-transform-input-bounds, .neuroglancer-coordinate-space-transform-output-bounds {
  white-space: nowrap;
  font-family: monospace;
  font-size: small;
  text-align: right;
  padding-left: 2px;
  padding-right: 2px;
  color: #aff;
}

.neuroglancer-coordinate-space-transform-input-bounds.neuroglancer-coordinate-space-transform-singleton:not(*[data-will-be-deleted="true"]) {
  color: #fa6;
}

.neuroglancer-coordinate-space-transform-widget input[data-is-valid="false"], .neuroglancer-coordinate-space-transform-widget input[data-is-valid="false"]::placeholder {
  text-decoration: solid underline red;
}

.neuroglancer-coordinate-space-transform-output-extend {
  grid-row: outputDim -1;
  grid-column: outputNames;
  display: flex;
  flex-direction: column;
}

.neuroglancer-coordinate-space-transform-output-extend input {
  align-self: stretch;
}

.neuroglancer-coordinate-space-transform-output-extend[data-is-active="false"] input {
  display: none;
}

.neuroglancer-coordinate-space-transform-output-extend[data-is-active="true"] .neuroglancer-icon {
  display: none;
}

.neuroglancer-coordinate-space-transform-output-extend .neuroglancer-icon {
  align-self: start;
}

.neuroglancer-coordinate-space-transform-translation-coeff {
  margin-left: 3px;
}

.neuroglancer-coordinate-space-transform-output-scale-container {
  margin-right: 3px;
}

.neuroglancer-coordinate-space-transform-input-scale-container,
.neuroglancer-coordinate-space-transform-input-scale-label {
  margin-bottom: 3px;
}

.neuroglancer-coordinate-space-transform-widget input,
.neuroglancer-coordinate-space-transform-output-name-container,
.neuroglancer-coordinate-space-transform-scale-container {
  border: 1px solid #333;
}

.neuroglancer-coordinate-space-transform-widget .neuroglancer-coordinate-space-transform-output-name,
.neuroglancer-coordinate-space-transform-widget .neuroglancer-coordinate-space-transform-scale {
  border: 0px;
}

.neuroglancer-coordinate-space-transform-output-name-container {
  display: flex;
  flex-direction: column;
  align-items: stretch;
  margin-right: 3px;
}

.neuroglancer-coordinate-space-transform-scale-container {
  display: flex;
  flex-direction: column;
  align-items: stretch;
}

.neuroglancer-coordinate-space-transform-scale-suggestion {
  font-family: monospace;
  cursor: pointer;
  border-radius: 20%;
  border: 1px solid #333;
  align-self: flex-end;
  margin: 2px;
  color: #aaa;
}

.neuroglancer-coordinate-space-transform-scale-suggestion svg {
  width: 16px;
  height: 16px;
  fill: transparent;
  stroke: #aaa;
  stroke-width: 2;
  stroke-linecap: round;
  stroke-linejoin: round;
}


.neuroglancer-coordinate-space-transform-scale-suggestion:hover {
  background-color: #db4437;
  border: 1px solid transparent;
  color: white;
}

.neuroglancer-coordinate-space-transform-scale-suggestion:hover svg {
  stroke: white;
}

.neuroglancer-coordinate-space-transform-widget-reset-buttons {
  grid-row: outputDim -1;
  grid-column: headerEnd / -1;
  justify-self: start;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-multiline-autocomplete {
  margin-top: 2px;
  padding: 2px;
  position: relative;
  background-color: #222;
}

.neuroglancer-multiline-autocomplete-hint, .neuroglancer-multiline-autocomplete-input {
  font-family: monospace;
  color: #fff;
  word-wrap: break-word;
}

.neuroglancer-multiline-autocomplete-input:focus {
  outline: 0px;
}

.neuroglancer-multiline-autocomplete-input:not(:focus):empty::before {
  content: attr(data-placeholder);
  color: #aaa;
}

.neuroglancer-multiline-autocomplete-hint {
  color: #aaa;
}

.neuroglancer-multiline-autocomplete-hint, .neuroglancer-multiline-autocomplete-input, .neuroglancer-multiline-autocomplete-completion {
  font-family: monospace;
  font-size: medium;
}

.neuroglancer-multiline-autocomplete-dropdown {
  color: #fff;
  background-color: #181818;
  position: absolute;
  display: block;
  outline: 0;
  margin: 0;
  padding: 0;
  left: 0px;
  right: 0px;
  text-align: left;
  cursor: default;
  border-style: solid;
  border-width: 1px;
  border-color: #aaa;
  overflow-y: scroll;
  word-wrap: break-word;
}

.neuroglancer-multiline-autocomplete-completion:nth-child(even):not(.neuroglancer-multiline-autocomplete-completion-active) {
  background-color: #2b2b2b;
}

.neuroglancer-multiline-autocomplete-completion:hover {
  outline: 1px solid #ddd;
}

.neuroglancer-multiline-autocomplete-completion-active {
  background-color: #666;
}

.neuroglancer-multiline-autocomplete-completion-with-description {
}

.neuroglancer-multiline-autocomplete-completion-description {
  font-style: italic;
  color: #f9f;
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-tab-view {
  display: flex;
  flex-direction: column;
}

.neuroglancer-tab-view-bar {
  display: block;
  background-color: #333;
  border-bottom: 1px solid white;
}

.neuroglancer-tab-label {
  display: inline-block;
  border-top: 1px solid transparent;
  border-left: 1px solid transparent;
  border-right: 1px solid transparent;
  border-bottom: none;
  margin-right: 4px;
  padding-top: 1px;
  padding-left: 2px;
  padding-right: 2px;
  font: 10pt sans-serif;
  font-weight: bold;
  cursor: pointer;
}
.neuroglancer-tab-label:hover {
  color: #daa520;
}

.neuroglancer-tab-label.neuroglancer-selected-tab-label {
  background-color: black;
  border-top: 1px solid white;
  border-left: 1px solid white;
  border-right: 1px solid white;
  border-bottom: 1px solid black;
  padding-bottom: 1px;
  margin-bottom: -1px;
}

.neuroglancer-stack-view {
  display: contents;
}

.neuroglancer-tab-view > .neuroglancer-stack-view > .neuroglancer-tab-content {
  flex: 1;
  flex-basis: 0px;
  height: 0px;
  min-height: 0px;
  padding: 2px;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.overlay {
  height: 100%;
  width: 100%;
  position: fixed;
  z-index: 99;
  top: 0;
  left: 0;
  background-color: rgba(0, 0, 0, 0.8);
}

.overlay-content {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
  background-color: white;
  z-index: 100;
  color: black;
  padding: 1em;
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-annotations-tab {
  display: flex;
  align-items: stretch;
  flex: 1;
  flex-direction: column;
}

.neuroglancer-annotation-list { 
  display: grid;
  grid-auto-rows: min-content;
  margin: 0px;
  padding: 0px;
  margin-top: 2px;
  overflow-y: auto;
  height: 0px;
  flex: 1;
  flex-basis: 0px;
  min-height: 0px;
}

.neuroglancer-annotation-list-entry {
  display: contents;
  cursor: pointer;
}

.neuroglancer-annotation-position {
  display: contents;
}

.neuroglancer-annotation-sublist {
  display: contents;
}

.neuroglancer-annotation-layer-view {
  display: flex;
  flex-direction: column;
  flex: 1;
  align-items: stretch;
}

.neuroglancer-annotation-list-header {
  display: contents;
}

.neuroglancer-annotation-list-header > * {
  position: sticky;
  top: 0px;
  background-color: black;
  padding-bottom: 2px;
}

.neuroglancer-annotation-coordinate {
  font-family: monospace;
  text-align: right;
}

.neuroglancer-annotation-icon {
  grid-column: symbol;
  padding-right: 5px;
}

.neuroglancer-annotation-description {
  grid-column: dim / -1;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
}

.neuroglancer-annotation-list-entry-delete {
  grid-column: delete;
  margin-left: 1ch;
  align-self: start;  
}

.neuroglancer-annotation-hover .neuroglancer-annotation-list-entry-highlight {
  background-color: #333;
}

.neuroglancer-annotation-selected .neuroglancer-annotation-list-entry-highlight {
  background-color: #939;
}

.neuroglancer-annotation-hover.neuroglancer-annotation-selected .neuroglancer-annotation-list-entry-highlight {
  background-color: #969;
}

.neuroglancer-tab-content.neuroglancer-annotation-details {
  flex: 0 0 auto;
  align-self: stretch;
  display: flex;
  flex-direction: column;

  -webkit-touch-callout: default;
  -webkit-user-select: text;
  -khtml-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  user-select: text;
}

.neuroglancer-annotation-details-title {
  display: flex;
  flex-direction: row;
  background-color: #03c;
  align-self: stretch;
  padding: 2px;
}

.neuroglancer-voxel-coordinates-link {
  cursor: pointer;
}

.neuroglancer-voxel-coordinates-link:hover {
  background-color: #db4437;
}

.neuroglancer-annotation-details-icon {
  display: inline-block;
  margin-right: 5px;
}

.neuroglancer-annotation-details-title-text {
  display: inline-block;
  flex: 1;
}

textarea.neuroglancer-annotation-details-description {
  align-self: stretch;
  background-color: #222;
  color: white;
  font: 10pt sans-serif;
  border: 0px;
  outline: 0px;
  resize: none;
  overflow-y: scroll;
}

div.neuroglancer-annotation-details-description {
  align-self: stretch;
  color: white;
  font: 10pt sans-serif;
}

.neuroglancer-annotation-details-position {
}

.neuroglancer-annotation-toolbox {
  display: flex;
  align-items: stretch;
}

.neuroglancer-annotation-segment-list {
}


.neuroglancer-annotation-segment-item {
  color: black;
  background-color: white;
}

.neuroglancer-annotations-view-dimension {
  font-family: monospace;
  text-align: right;
  padding-left: 2ch;
}

.neuroglancer-annotations-view-dimension-name {
  color: #ff6;
}

.neuroglancer-annotations-view-dimension-scale {
  color: #bbb;
}

.neuroglancer-annotations-view-dimension-scale:not(:empty)::before {
  content: "(";
  margin-left: 1ch;
}

.neuroglancer-annotations-view-dimension-scale:not(:empty)::after {
  content: ")";
}

.neuroglancer-annotation-relationship-label, .neuroglancer-annotation-property-label {
  margin-right: 5px;
}


.neuroglancer-selected-annotation-details-position-grid {
  display: grid;
  grid-auto-rows: auto;
  font-family: monospace;
  font-size: medium;
  grid-auto-flow: dense;
}

.neuroglancer-selected-annotation-details-icon {
  grid-rows: 1 / -1;
  grid-column: icon;
  align-self: start;
}

.neuroglancer-selected-annotation-details-delete {
  grid-rows: 1 / -1;
  grid-column: delete;
  align-self: start;
}

.neuroglancer-selected-annotation-details-position-dim {
  color: #ff6;
  margin-left: 1ch;
}

.neuroglancer-selected-annotation-details-position-coord {
  text-align: right;
  margin-left: 0.5ch;
}

.neuroglancer-related-segment-list-title {
  font-family: sans-serif;
  font-size: small;
 background-color: #333;
}

.neuroglancer-related-segment-list-header {
  display: flex;
  flex-direction: row;
}

.neuroglancer-related-segment-list {
/*  border: 1px solid #333;
  margin: 3px;*/
}

.neuroglancer-annotation-property {
  display: flex;
  flex-direction: row;
  font-family: sans-serif;
  font-size: small;
}

.neuroglancer-annotation-property-label {
  color: #999;
}

.neuroglancer-annotation-property-value {
  text-align: right;
  font-family: monospace;
  font-size: medium;
  flex: 1;
}

input.neuroglancer-segment-list-entry-id {
  outline: 0px;
  border: 0px;
  text-align: left;
}

.neuroglancer-segment-list-entry-new > .neuroglancer-segment-list-entry-copy {
  visibility: hidden;
}

.neuroglancer-segment-list-entry-new > input[type="checkbox"] {
  visibility: hidden;
}

.neuroglancer-selection-annotation-status {
  font-family: sans-serif;
  font-size: small;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-channel-dimensions-widget-dim {
  display: contents;
}

.neuroglancer-channel-dimensions-widget {
  display: grid;
  grid-template-columns: [name] min-content [lower] min-content [upper] min-content;
}

.neuroglancer-channel-dimensions-widget-name-container {
  grid-column: name;
}

.neuroglancer-channel-dimensions-widget-name[data-is-valid="false"],
.neuroglancer-channel-dimensions-widget-name::placeholder {
  text-decoration: solid underline red;
}

.neuroglancer-channel-dimensions-widget-name {
  background-color: transparent;
  border-color: transparent;
  box-shadow: none;
  border: 0;
  margin: 0;
  font-family: monospace;
  font-size: medium;
  color: #ff6;
}

.neuroglancer-channel-dimensions-widget-name,
.neuroglancer-channel-dimensions-widget-lower,
.neuroglancer-channel-dimensions-widget-upper {
  font-family: monospace;
}


.neuroglancer-channel-dimensions-widget-lower {
  grid-column: lower;
  text-align: right;
}

.neuroglancer-channel-dimensions-widget-upper {
  grid-column: upper;
  text-align: right;
}

.neuroglancer-channel-dimensions-widget-lower,
.neuroglancer-channel-dimensions-widget-upper {
  text-align: right;
  margin-left: 1ch;
  user-select: text;
}

.neuroglancer-channel-dimensions-widget-upper::after {
  content: ")";
}

.neuroglancer-channel-dimensions-widget-lower::after {
  content: ",";
}

.neuroglancer-channel-dimensions-widget-lower::before {
  content: "[";
}

.neuroglancer-channel-dimensions-widget-upper::after,
.neuroglancer-channel-dimensions-widget-lower::before,
.neuroglancer-channel-dimensions-widget-lower::after {
  color: #999;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.range-slider {
  display: flex;
  flex-direction: row;
  white-space: nowrap;
  justify-content: space-between;
}

.range-slider input[type='range'] {
}


/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-render-scale-widget {
  margin-top: 2px;
  margin-bottom: 2px;
  display: flex;
  flex-direction: row;
  align-items: center;
}

.neuroglancer-render-scale-widget-prompt {
  white-space: nowrap;
}

.neuroglancer-render-scale-widget > canvas {
  height: 36px;
  flex: 1;
  flex-basis: 0px;
  width: 0px;
}


.neuroglancer-render-scale-widget-legend {
  width: 10ch;
  font-size: 11px;
  text-align: right;
}

.neuroglancer-render-scale-widget-legend > div {
  height: 12px;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-shader-code-widget.invalid-input {
  border: 1px solid red;
}

.neuroglancer-shader-code-widget.valid-input {
  border: 1px solid green;
}

.neuroglancer-shader-code-widget {
  border: 1px solid transparent;
}

/* BASICS */

.CodeMirror {
  /* Set height, width, borders, and global font properties here */
  font-family: monospace;
  height: 300px;
  color: black;
  direction: ltr;
}

/* PADDING */

.CodeMirror-lines {
  padding: 4px 0; /* Vertical padding around content */
}
.CodeMirror pre.CodeMirror-line,
.CodeMirror pre.CodeMirror-line-like {
  padding: 0 4px; /* Horizontal padding of content */
}

.CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  background-color: white; /* The little square between H and V scrollbars */
}

/* GUTTER */

.CodeMirror-gutters {
  border-right: 1px solid #ddd;
  background-color: #f7f7f7;
  white-space: nowrap;
}
.CodeMirror-linenumbers {}
.CodeMirror-linenumber {
  padding: 0 3px 0 5px;
  min-width: 20px;
  text-align: right;
  color: #999;
  white-space: nowrap;
}

.CodeMirror-guttermarker { color: black; }
.CodeMirror-guttermarker-subtle { color: #999; }

/* CURSOR */

.CodeMirror-cursor {
  border-left: 1px solid black;
  border-right: none;
  width: 0;
}
/* Shown when moving in bi-directional text */
.CodeMirror div.CodeMirror-secondarycursor {
  border-left: 1px solid silver;
}
.cm-fat-cursor .CodeMirror-cursor {
  width: auto;
  border: 0 !important;
  background: #7e7;
}
.cm-fat-cursor div.CodeMirror-cursors {
  z-index: 1;
}
.cm-fat-cursor-mark {
  background-color: rgba(20, 255, 20, 0.5);
  -webkit-animation: blink 1.06s steps(1) infinite;
  -moz-animation: blink 1.06s steps(1) infinite;
  animation: blink 1.06s steps(1) infinite;
}
.cm-animate-fat-cursor {
  width: auto;
  border: 0;
  -webkit-animation: blink 1.06s steps(1) infinite;
  -moz-animation: blink 1.06s steps(1) infinite;
  animation: blink 1.06s steps(1) infinite;
  background-color: #7e7;
}
@-moz-keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}
@-webkit-keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}
@keyframes blink {
  0% {}
  50% { background-color: transparent; }
  100% {}
}

/* Can style cursor different in overwrite (non-insert) mode */
.CodeMirror-overwrite .CodeMirror-cursor {}

.cm-tab { display: inline-block; text-decoration: inherit; }

.CodeMirror-rulers {
  position: absolute;
  left: 0; right: 0; top: -50px; bottom: 0;
  overflow: hidden;
}
.CodeMirror-ruler {
  border-left: 1px solid #ccc;
  top: 0; bottom: 0;
  position: absolute;
}

/* DEFAULT THEME */

.cm-s-default .cm-header {color: blue;}
.cm-s-default .cm-quote {color: #090;}
.cm-negative {color: #d44;}
.cm-positive {color: #292;}
.cm-header, .cm-strong {font-weight: bold;}
.cm-em {font-style: italic;}
.cm-link {text-decoration: underline;}
.cm-strikethrough {text-decoration: line-through;}

.cm-s-default .cm-keyword {color: #708;}
.cm-s-default .cm-atom {color: #219;}
.cm-s-default .cm-number {color: #164;}
.cm-s-default .cm-def {color: #00f;}
.cm-s-default .cm-variable,
.cm-s-default .cm-punctuation,
.cm-s-default .cm-property,
.cm-s-default .cm-operator {}
.cm-s-default .cm-variable-2 {color: #05a;}
.cm-s-default .cm-variable-3, .cm-s-default .cm-type {color: #085;}
.cm-s-default .cm-comment {color: #a50;}
.cm-s-default .cm-string {color: #a11;}
.cm-s-default .cm-string-2 {color: #f50;}
.cm-s-default .cm-meta {color: #555;}
.cm-s-default .cm-qualifier {color: #555;}
.cm-s-default .cm-builtin {color: #30a;}
.cm-s-default .cm-bracket {color: #997;}
.cm-s-default .cm-tag {color: #170;}
.cm-s-default .cm-attribute {color: #00c;}
.cm-s-default .cm-hr {color: #999;}
.cm-s-default .cm-link {color: #00c;}

.cm-s-default .cm-error {color: #f00;}
.cm-invalidchar {color: #f00;}

.CodeMirror-composing { border-bottom: 2px solid; }

/* Default styles for common addons */

div.CodeMirror span.CodeMirror-matchingbracket {color: #0b0;}
div.CodeMirror span.CodeMirror-nonmatchingbracket {color: #a22;}
.CodeMirror-matchingtag { background: rgba(255, 150, 0, .3); }
.CodeMirror-activeline-background {background: #e8f2ff;}

/* STOP */

/* The rest of this file contains styles related to the mechanics of
   the editor. You probably shouldn't touch them. */

.CodeMirror {
  position: relative;
  overflow: hidden;
  background: white;
}

.CodeMirror-scroll {
  overflow: scroll !important; /* Things will break if this is overridden */
  /* 30px is the magic margin used to hide the element's real scrollbars */
  /* See overflow: hidden in .CodeMirror */
  margin-bottom: -30px; margin-right: -30px;
  padding-bottom: 30px;
  height: 100%;
  outline: none; /* Prevent dragging from highlighting the element */
  position: relative;
}
.CodeMirror-sizer {
  position: relative;
  border-right: 30px solid transparent;
}

/* The fake, visible scrollbars. Used to force redraw during scrolling
   before actual scrolling happens, thus preventing shaking and
   flickering artifacts. */
.CodeMirror-vscrollbar, .CodeMirror-hscrollbar, .CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  position: absolute;
  z-index: 6;
  display: none;
}
.CodeMirror-vscrollbar {
  right: 0; top: 0;
  overflow-x: hidden;
  overflow-y: scroll;
}
.CodeMirror-hscrollbar {
  bottom: 0; left: 0;
  overflow-y: hidden;
  overflow-x: scroll;
}
.CodeMirror-scrollbar-filler {
  right: 0; bottom: 0;
}
.CodeMirror-gutter-filler {
  left: 0; bottom: 0;
}

.CodeMirror-gutters {
  position: absolute; left: 0; top: 0;
  min-height: 100%;
  z-index: 3;
}
.CodeMirror-gutter {
  white-space: normal;
  height: 100%;
  display: inline-block;
  vertical-align: top;
  margin-bottom: -30px;
}
.CodeMirror-gutter-wrapper {
  position: absolute;
  z-index: 4;
  background: none !important;
  border: none !important;
}
.CodeMirror-gutter-background {
  position: absolute;
  top: 0; bottom: 0;
  z-index: 4;
}
.CodeMirror-gutter-elt {
  position: absolute;
  cursor: default;
  z-index: 4;
}
.CodeMirror-gutter-wrapper ::selection { background-color: transparent }
.CodeMirror-gutter-wrapper ::-moz-selection { background-color: transparent }

.CodeMirror-lines {
  cursor: text;
  min-height: 1px; /* prevents collapsing before first draw */
}
.CodeMirror pre.CodeMirror-line,
.CodeMirror pre.CodeMirror-line-like {
  /* Reset some styles that the rest of the page might have set */
  -moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;
  border-width: 0;
  background: transparent;
  font-family: inherit;
  font-size: inherit;
  margin: 0;
  white-space: pre;
  word-wrap: normal;
  line-height: inherit;
  color: inherit;
  z-index: 2;
  position: relative;
  overflow: visible;
  -webkit-tap-highlight-color: transparent;
  -webkit-font-variant-ligatures: contextual;
  font-variant-ligatures: contextual;
}
.CodeMirror-wrap pre.CodeMirror-line,
.CodeMirror-wrap pre.CodeMirror-line-like {
  word-wrap: break-word;
  white-space: pre-wrap;
  word-break: normal;
}

.CodeMirror-linebackground {
  position: absolute;
  left: 0; right: 0; top: 0; bottom: 0;
  z-index: 0;
}

.CodeMirror-linewidget {
  position: relative;
  z-index: 2;
  padding: 0.1px; /* Force widget margins to stay inside of the container */
}

.CodeMirror-widget {}

.CodeMirror-rtl pre { direction: rtl; }

.CodeMirror-code {
  outline: none;
}

/* Force content-box sizing for the elements where we expect it */
.CodeMirror-scroll,
.CodeMirror-sizer,
.CodeMirror-gutter,
.CodeMirror-gutters,
.CodeMirror-linenumber {
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}

.CodeMirror-measure {
  position: absolute;
  width: 100%;
  height: 0;
  overflow: hidden;
  visibility: hidden;
}

.CodeMirror-cursor {
  position: absolute;
  pointer-events: none;
}
.CodeMirror-measure pre { position: static; }

div.CodeMirror-cursors {
  visibility: hidden;
  position: relative;
  z-index: 3;
}
div.CodeMirror-dragcursors {
  visibility: visible;
}

.CodeMirror-focused div.CodeMirror-cursors {
  visibility: visible;
}

.CodeMirror-selected { background: #d9d9d9; }
.CodeMirror-focused .CodeMirror-selected { background: #d7d4f0; }
.CodeMirror-crosshair { cursor: crosshair; }
.CodeMirror-line::selection, .CodeMirror-line > span::selection, .CodeMirror-line > span > span::selection { background: #d7d4f0; }
.CodeMirror-line::-moz-selection, .CodeMirror-line > span::-moz-selection, .CodeMirror-line > span > span::-moz-selection { background: #d7d4f0; }

.cm-searching {
  background-color: #ffa;
  background-color: rgba(255, 255, 0, .4);
}

/* Used to force a border model for a node */
.cm-force-border { padding-right: .1px; }

@media print {
  /* Hide the cursor when printing */
  .CodeMirror div.CodeMirror-cursors {
    visibility: hidden;
  }
}

/* See issue #2901 */
.cm-tab-wrap-hack:after { content: ''; }

/* Help users use markselection to safely style text background */
span.CodeMirror-selectedtext { background: none; }

/* The lint marker gutter */
.CodeMirror-lint-markers {
  width: 16px;
}

.CodeMirror-lint-tooltip {
  background-color: #ffd;
  border: 1px solid black;
  border-radius: 4px 4px 4px 4px;
  color: black;
  font-family: monospace;
  font-size: 10pt;
  overflow: hidden;
  padding: 2px 5px;
  position: fixed;
  white-space: pre;
  white-space: pre-wrap;
  z-index: 100;
  max-width: 600px;
  opacity: 0;
  transition: opacity .4s;
  -moz-transition: opacity .4s;
  -webkit-transition: opacity .4s;
  -o-transition: opacity .4s;
  -ms-transition: opacity .4s;
}

.CodeMirror-lint-mark-error, .CodeMirror-lint-mark-warning {
  background-position: left bottom;
  background-repeat: repeat-x;
}

.CodeMirror-lint-mark-error {
  background-image:
  url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAADCAYAAAC09K7GAAAAAXNSR0IArs4c6QAAAAZiS0dEAP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sJDw4cOCW1/KIAAAAZdEVYdENvbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVBXgQ4XAAAAHElEQVQI12NggIL/DAz/GdA5/xkY/qPKMDAwAADLZwf5rvm+LQAAAABJRU5ErkJggg==")
  ;
}

.CodeMirror-lint-mark-warning {
  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAADCAYAAAC09K7GAAAAAXNSR0IArs4c6QAAAAZiS0dEAP8A/wD/oL2nkwAAAAlwSFlzAAALEwAACxMBAJqcGAAAAAd0SU1FB9sJFhQXEbhTg7YAAAAZdEVYdENvbW1lbnQAQ3JlYXRlZCB3aXRoIEdJTVBXgQ4XAAAAMklEQVQI12NkgIIvJ3QXMjAwdDN+OaEbysDA4MPAwNDNwMCwiOHLCd1zX07o6kBVGQEAKBANtobskNMAAAAASUVORK5CYII=");
}

.CodeMirror-lint-marker-error, .CodeMirror-lint-marker-warning {
  background-position: center center;
  background-repeat: no-repeat;
  cursor: pointer;
  display: inline-block;
  height: 16px;
  width: 16px;
  vertical-align: middle;
  position: relative;
}

.CodeMirror-lint-message-error, .CodeMirror-lint-message-warning {
  padding-left: 18px;
  background-position: top left;
  background-repeat: no-repeat;
}

.CodeMirror-lint-marker-error, .CodeMirror-lint-message-error {
  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAAHlBMVEW7AAC7AACxAAC7AAC7AAAAAAC4AAC5AAD///+7AAAUdclpAAAABnRSTlMXnORSiwCK0ZKSAAAATUlEQVR42mWPOQ7AQAgDuQLx/z8csYRmPRIFIwRGnosRrpamvkKi0FTIiMASR3hhKW+hAN6/tIWhu9PDWiTGNEkTtIOucA5Oyr9ckPgAWm0GPBog6v4AAAAASUVORK5CYII=");
}

.CodeMirror-lint-marker-warning, .CodeMirror-lint-message-warning {
  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAANlBMVEX/uwDvrwD/uwD/uwD/uwD/uwD/uwD/uwD/uwD6twD/uwAAAADurwD2tQD7uAD+ugAAAAD/uwDhmeTRAAAADHRSTlMJ8mN1EYcbmiixgACm7WbuAAAAVklEQVR42n3PUQqAIBBFUU1LLc3u/jdbOJoW1P08DA9Gba8+YWJ6gNJoNYIBzAA2chBth5kLmG9YUoG0NHAUwFXwO9LuBQL1giCQb8gC9Oro2vp5rncCIY8L8uEx5ZkAAAAASUVORK5CYII=");
}

.CodeMirror-lint-marker-multiple {
  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAcAAAAHCAMAAADzjKfhAAAACVBMVEUAAAAAAAC/v7914kyHAAAAAXRSTlMAQObYZgAAACNJREFUeNo1ioEJAAAIwmz/H90iFFSGJgFMe3gaLZ0od+9/AQZ0ADosbYraAAAAAElFTkSuQmCC");
  background-repeat: no-repeat;
  background-position: right bottom;
  width: 100%; height: 100%;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-noselect {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.neuroglancer-select-text {
  -webkit-touch-callout: default;
  -webkit-user-select: text;
  -khtml-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  user-select: text;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.segmentation-dropdown {
  display: flex;
  flex-direction: column;
  overflow-y: auto;
}

.segmentation-dropdown .neuroglancer-shader-code-widget {
  height: 6em;
  width: 60ch;
}

.neuroglancer-segmentation-dropdown-skeleton-shader-header {
  display: flex;
  flex-direction: row;
  align-items: center;
}

.neuroglancer-segmentation-layer-skeleton-shader-overlay .neuroglancer-shader-code-widget {
  width: 80vw;
  height: 80vh;
}

.neuroglancer-segment-list {
  position: relative;
  overflow-y: auto;
  overflow-x: auto;
}

.neuroglancer-segment-display-tab {
  display: flex;
  flex-direction: column;
}

.neuroglancer-segment-list-entry, .neuroglancer-selection-details-segment {
  display: flex;
  flex-direction: row;
  align-items: center;
}

.neuroglancer-segment-list-entry[data-selected="true"],
.neuroglancer-selection-details-segment[data-selected="true"] {
  background-color: #222;
}

.neuroglancer-segment-list-entry-id,
.neuroglancer-selection-details-segment-id,
.neuroglancer-selection-details-segment-mapped-id {
  display: inline-block;
  font-family: monospace;
  font-size: medium;
  flex-shrink: 0;
  text-align: right;
  color: black;
  background-color: white;
  user-select: text;
  width: var(--neuroglancer-segment-list-width);
}

.neuroglancer-segment-list-entry-name {
  display: inline-block;
  font-family: monospace;
  font-size: medium;
  user-select: text;
  white-space: nowrap;
}

.neuroglancer-segment-list-query {
  background-color: #151515;
  color: white;
  font-family: monospace;
  font-size: medium;
  border: 2px solid #333;
  padding: 2px;
  outline: 0px;
}

.neuroglancer-segment-list-query::placeholder {
  color: #aaa;
}

.neuroglancer-segment-list-status {
  display: flex;
  flex-direction: row;
  align-items: center;
  background-color: #333;
}

.neuroglancer-segment-list-entry-copy {
  visibility: hidden;
}

.neuroglancer-segment-list-entry:hover .neuroglancer-segment-list-entry-copy {
  visibility: visible;
}

.neuroglancer-segment-list-entry[data-visible-list]::before {
  content: " ";
  display: inline-block;
  width: 2px;
  /* prevent collapsing if name is too long */
  flex-shrink: 0;
  margin-right: -2px;
  align-self: stretch;
  margin-bottom: 2px;
  background-color: #999;
}

.neuroglancer-selection-details-segment-description {
  font-family: sans-serif;
  font-size: small;
}

.neuroglancer-segment-list-entry-filter {
  visibility: hidden;
}

.neuroglancer-segment-list-entry:hover .neuroglancer-segment-list-entry-filter {
  visibility: visible;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-single-mesh-dropdown .neuroglancer-single-mesh-attribute-widget {
  max-height: 6em;
  margin-bottom: 8px;
}

.neuroglancer-single-mesh-dropdown .neuroglancer-shader-code-widget {
  height: 6em;
  width: 60ch;
  border: 1px solid transparent;
}

.neuroglancer-single-mesh-dropdown-top-row {
  display: flex;
  flex-direction: row;
  align-items: center;
}

.neuroglancer-single-mesh-shader-overlay .neuroglancer-shader-code-widget {
  width: 80vw;
  height: 80vh;
}

.neuroglancer-single-mesh-attribute-widget {
  word-wrap: break-word;
  overflow-y: auto;
  display: grid;
  grid-template-columns: [type] auto [name] auto [range] auto;
}

.neuroglancer-single-mesh-layer-shader-overlay .neuroglancer-single-mesh-attribute-widget {
  max-height: 20vh;
}

.neuroglancer-single-mesh-attribute {
  font-family: monospace;
  display: contents;
}

.neuroglancer-single-mesh-attribute-type {
  color: #c6c;
  grid-column: type;
}

.neuroglancer-single-mesh-attribute-name {
  margin-left: 1ch;
  color: white;
  grid-column: name;
  user-select: text;
}

.neuroglancer-single-mesh-attribute-range {
  grid-column: range;
  margin-left: 1ch;
  white-space: nowrap;
  color: #999;
  grid-column: range;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-annotation-point-list-dropdown {
  display: flex;
}

.neuroglancer-annotation-shader-property {
  white-space: pre;
  font-family: monospace;
  font-size: medium;
}


.neuroglancer-annotation-shader-property-type {
  color: #c6c;
  margin-right: 1ch;
}

.neuroglancer-annotation-shader-property-identifier {
  user-select: text;
}

.neuroglancer-annotation-shader-property-identifier::after {
  content: "()";
  color: #999;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-viewer {
  outline: 0px;
}

.neuroglancer-viewer-context-menu > label {
  display: flex;
  flex-direction: row;
  justify-content: space-between;
}

.neuroglancer-viewer-context-menu > label > input::before {
  flex: 1;
  content: " ";
}

.neuroglancer-viewer-context-menu > label > input {
  margin-left: 5px;
  width: 11ch;
}

.neuroglancer-viewer-context-menu-limit-widget > input {
  width: 11ch;
}

.neuroglancer-viewer-top-row {
  display: flex;
  flex-direction: row;
  align-items: center;
  background-color: #222;
  margin-bottom: 3px;
  padding-right: 2px;
}

.neuroglancer-viewer-side-panel {
  display: flex;
  flex-direction: column;
  align-items: stretch;
  max-width: 80%;
  color: #fff;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-data-panel-layout-controls {
  position: absolute;
  top: 2px;
  right: 2px;
  display: flex;
  align-items: start;
}

.neuroglancer-data-panel-layout-controls > button {
  display: flex;
  margin: 2px;
  border: 0px;
  background-color: rgba(0, 0, 0, 0.5);
  flex-direction: row;
  align-self: top;
  align-items: center;
  justify-content: center;
  border: 1px solid white;
  font-size: 15px;
  width: 18px;
  height: 18px;
  color: white;
}

.neuroglancer-data-panel-layout-controls > button > div {
  display: inline-block;
  text-align: center;
  line-height: 12px;
  width: 15px;
  height: 15px;
}

.neuroglancer-data-panel-layout-controls > button:hover {
  color: #6ff;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

label.perspective-panel-show-slice-views {
  pointer-events: none;
  position: absolute;
  right: 2px;
  bottom: 2px;
  font-family: sans-serif;
  font-size: small;
}

input.perspective-panel-show-slice-views {
  pointer-events: all;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-rendered-data-panel {
  cursor: crosshair;
  position: relative;
  outline: 0;
  touch-action: none;
  color: #fff;
  text-align:left;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-display-dimensions-widget {
  position: absolute;
  cursor: default;
  top: 2px;
  left: 2px;
  background-color: rgba(0, 0, 0, 0.3);
}

.neuroglancer-display-dimensions-widget-dimension-grid {
  display: grid;
  grid-template-rows: 0fr 0fr 0fr 0fr;
  grid-template-columns: 0fr 0fr 0fr;
}

.neuroglancer-display-dimensions-widget input {
  outline: 0px;
  background-color: transparent;
  border-color: transparent;
  box-shadow: none;
  margin: 0;
  border: 0;
  padding: 2px;
}

.neuroglancer-display-dimensions-widget input,
.neuroglancer-display-dimensions-widget {
  font-family: monospace;
  color: white;
}

.neuroglancer-display-dimensions-widget-dimension:hover {
  outline: 1px solid black;
}

.neuroglancer-display-dimensions-widget-name[data-is-valid="false"] {
  text-decoration: solid underline red;
}

.neuroglancer-display-dimensions-widget-scale-factor {
  text-align: right;
  align-items: end;
  display: inline-block;
  white-space: nowrap;
  margin-left: 2px;
}

.neuroglancer-display-dimensions-widget-scale {
  display: inline-block;
  white-space: nowrap;
  padding-left: 10px;
}

.neuroglancer-display-dimensions-widget-scale:not(:empty)::before {
  content: "(";
}

.neuroglancer-display-dimensions-widget-scale:not(:empty)::after {
  content: ")";
}

.neuroglancer-display-dimensions-widget-scale-factor::after {
  content: "×";
}

.neuroglancer-display-dimensions-widget-scale {
  text-align: right;
  margin-left: 5px;
}

.neuroglancer-display-dimensions-widget:not(:hover):not([data-active="true"]) .neuroglancer-display-dimensions-widget-scale-factor,
.neuroglancer-display-dimensions-widget:not(:hover):not([data-active="true"]) .neuroglancer-display-dimensions-widget-scale,
.neuroglancer-display-dimensions-widget:not(:hover):not([data-active="true"]) .neuroglancer-display-dimensions-widget-default,
.neuroglancer-display-dimensions-widget:not(:hover):not([data-active="true"]) .neuroglancer-depth-range-widget-grid,
.neuroglancer-display-dimensions-widget:not(:hover):not([data-active="true"]) .neuroglancer-depth-range-relative-checkbox-label {
  display: none;
}

.neuroglancer-display-dimensions-widget-dimension[data-is-modified="true"] .neuroglancer-display-dimensions-widget-scale-factor,
.neuroglancer-display-dimensions-widget-dimension[data-is-modified="true"] .neuroglancer-display-dimensions-widget-scale {
  visibility: hidden;
}

.neuroglancer-display-dimensions-widget *:focus {
  outline: 0px;
}

.neuroglancer-display-dimensions-widget-default {
  grid-row: 4;
  grid-column-start: 1;
  grid-column-end: 3;
  display: inline-block;
  white-space: nowrap;
}

.neuroglancer-display-dimensions-widget-default input {
  margin-right: 3px;
}

.neuroglancer-depth-range-widget-grid {
  margin-top: 1em;
  display: grid;
  display: grid;
  grid-template-columns: 0fr 0fr 0fr;
  grid-auto-rows: 0fr;
}

.neuroglancer-depth-range-widget-dimension-names:not(:empty)::before {
  content: "(";
}

.neuroglancer-depth-range-widget-dimension-names:not(:empty)::after {
  content: ")";
}

.neuroglancer-depth-range-widget-dimension-names {
  margin-left: 1ch;
  white-space: nowrap;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.describe-key-bindings {
  overflow-x: hidden;
}

.describe-key-bindings-container {
  display: grid;
  overflow-y: scroll;
  max-height: 80vh;
  overflow-x: hidden;
  grid-template-columns: [dt-start] 0fr [dt-end] 1em [dd-start] 1fr [dd-end];
  grid-auto-rows: auto;
}

.describe-key-bindings-container h2 {
  grid-column-start: 1;
  grid-column-end: -1;
}

.describe-key-bindings .dt {
  grid-column: dt-start;
  font-family: monospace;
  font-weight: bold;
  display: inline-block;
}

.describe-key-bindings .dd {
  grid-column: dd-start;
  display: inline-block;
}

/**
 * @license
 * Copyright 2017 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


.neuroglancer-layout-split-drop-zone {
  position: absolute;
  z-index: 1000;
}

.neuroglancer-drag-over {
  background-color: rgba(0, 0, 255, 0.5);
}

.neuroglancer-stack-layout-row > .neuroglancer-stack-layout-drop-placeholder {
  padding-left: 4px;
  padding-right: 4px;
  width: 1px;
}

.neuroglancer-stack-layout-column > .neuroglancer-stack-layout-drop-placeholder {
  padding-top: 4px;
  padding-bottom: 4px;
  height: 1px;
}

.neuroglancer-stack-layout-drop-placeholder {
  background-clip: content-box;
  background-color: #666;
  z-index: 1;
}

.neuroglancer-stack-layout-drop-placeholder.neuroglancer-drag-over {
  background-clip: border-box;
  background-color: rgba(128, 128, 255, 0.5);
}

.neuroglancer-stack-layout-drop-placeholder:first-child, .neuroglancer-stack-layout-drop-placeholder:last-child {
  display: none;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-panel {
  flex: 1;
}

.neuroglancer-show-panel-borders .neuroglancer-panel {
  border-style: solid;
  border-color: black;
  border-width: 2px;
}

.neuroglancer-panel:focus-within {
  border-color: white;
}

.neuroglancer-layer-group-viewer {
  outline: 0px;
}

.neuroglancer-layer-group-viewer-context-menu {
  flex-direction: column;
  align-items: stretch;
}

.neuroglancer-layer-group-viewer-context-menu label {
  display: flex;
  flex-direction: row;
  justify-content: space-between;
}

.neuroglancer-layer-group-viewer-context-menu select::before {
  flex: 1;
  content: " ";
}

.neuroglancer-layer-group-viewer-context-menu select {
  margin-left: 5px;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-layer-panel {
  width: 100%;
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
  background-color: black;
  align-items: stretch;
  margin-left: -5px;
  margin-top: -1px;
}

.neuroglancer-layer-item {
  position: relative;
  display: flex;
  align-items: center;
  color: white;
  cursor: pointer;
  border-width: 1px;
  border-style: solid;
  border-color: #ccc;
  font-family: sans-serif;
  font-size: 10pt;
  background-color: black;
  padding: 1px;
}

.neuroglancer-layer-item, .neuroglancer-layer-add-button {
  margin: 1px;
  margin-left: 5px;
}

.neuroglancer-layer-item[data-selected=true] {
  border-color: #8c8;
}

.neuroglancer-layer-item[data-selected=true]:hover {
  border-color: #3c3;
}

.neuroglancer-layer-item:hover {
  border-color: #daa520;
}

.neuroglancer-layer-item[data-pick="true"] .neuroglancer-layer-item-label {
  background-color: #939;
}

.neuroglancer-layer-item-label {
  display: inline-block;
  position: relative;
  background-color: #222;
  padding-right: 3px;
}

.neuroglancer-layer-item-number {
  display: inline-block;
  background-color: /*#daa520*/ #9a7518;
  font-weight: bold;
  padding-left: 1px;
  padding-right: 1px;
}

.neuroglancer-layer-panel[data-show-hover-values="true"] .neuroglancer-layer-item-value {
  display: inline-block;
  font-family: monospace;
  font-size: medium;
  max-width: 50ch;
  margin-left: 1ch;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipses;
  text-align: center;
}

.neuroglancer-layer-panel[data-show-hover-values="false"] .neuroglancer-layer-item-value { 
  display:none;
}

.neuroglancer-layer-item[data-visible=false] .neuroglancer-layer-item-label {
  text-decoration: line-through;
}

.neuroglancer-layer-item[data-visible=false] {
  color: #bbb;
}

.neuroglancer-layer-panel-drop-zone {
  display: inline-block;
  flex: 1;
}

.neuroglancer-layer-item-visible-progress, .neuroglancer-layer-item-prefetch-progress {
  position: absolute;
  left: 0px;
  height: 2px;
  background-color: #666;
}

.neuroglancer-layer-item-visible-progress {
  top: 0px;
}

.neuroglancer-layer-item-prefetch-progress {
  bottom: 0px;
}

/**
 * @license
 * Copyright 2017 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-position-widget {
  display: inline-flex;
  align-items: center;
}

/* In Firefox, disabled input elements block all pointer events, even on parent elements, and that
interferes with our click handling.  Setting `pointer-events: none;` avoids that problem. */
.neuroglancer-position-widget input:disabled {
  pointer-events: none;
}

.neuroglancer-position-widget .neuroglancer-copy-button:first-child {
  display: none;
}

.neuroglancer-position-dimension-coordinate, .neuroglancer-position-dimension-name, .neuroglancer-position-dimension-scale {
  background-color: transparent;
  border-color: transparent;
  box-shadow: none;
  border: 0;
  margin: 0;
  font-family: monospace;
  font-size: medium;
}

.neuroglancer-position-dimension-coordinate {
  color: white;
}

.neuroglancer-position-widget input:invalid, .neuroglancer-position-widget input::placeholder, .neuroglancer-position-widget input[data-is-valid="false"] {
  text-decoration: solid underline red;
}

.neuroglancer-position-widget *:focus {
  outline: 0px;
}

.neuroglancer-position-dimension[data-dropdown-visible="true"]::after {
  content: "";
  display: block;
  left: 0px;
  right: 0px;
  bottom: -1px;
  position: absolute;
  border-bottom: 1px solid black;
  z-index: 100;
  height: 0px;
}

.neuroglancer-position-dimension-dropdown {
  position: absolute;
  min-width: calc(100% + 2px);
  border: 1px solid #aaa;
  box-sizing: border-box;
  padding: 2px;
  left: -1px;
  z-index: 100;
  background-color: black;
  display: grid;
  grid-template-areas:
    "labels . graph";
  grid-template-rows: 1fr;
  grid-template-columns: 0fr 3px 0fr;
}

.neuroglancer-position-dimension-dropdown:focus {
  outline: 0px;
}

.neuroglancer-position-dimension-dropdown-lowerbound,
.neuroglancer-position-dimension-dropdown-upperbound,
.neuroglancer-position-dimension-dropdown-hoverposition {
  grid-area: labels;
  text-align: right;
}

.neuroglancer-position-dimension-dropdown canvas {
  margin-top: 0.5em;
  margin-bottom: 0.5em;
  grid-area: graph;
}

.neuroglancer-position-dimension:focus-within,
.neuroglancer-position-dimension[data-dropdown-visible="true"] {
  border: 1px solid #aaa;
  background-color: black;
}

.neuroglancer-position-dimension {
  border: 1px solid transparent;
  position: relative;
  display: inline-block;
  font-family: monospace;
  font-size: medium;
  white-space: nowrap;
}

.neuroglancer-position-dimension-name {
  color: #ff6;
  padding-left: 2px;
}

.neuroglancer-position-dimension-scale-container[data-is-empty="false"]::before {
  content: "(";
  color: #aaa;
}

.neuroglancer-position-dimension-scale-container[data-is-empty="false"]::after {
  content: ")";
  color: #aaa;
}

.neuroglancer-position-dimension-scale-container[data-is-empty="false"] {
  margin-right: 5px;
}

.neuroglancer-position-dimension-scale {
   color: #bbb;
}

.neuroglancer-position-dimension-scale-container {
  margin-right: 2px;
  margin-left: 4px;
}

.neuroglancer-mouse-position-widget {
  margin-left: 1ch;
  vertical-align: center;
  font-family: monospace;
  font-size: medium;
  color: orange;
  white-space: pre;
}

/**
 * @license
 * Copyright 2017 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-context-menu {
  position: absolute;
  z-index: 100;
  border: 1px solid white;
  background-color: black;
  box-shadow: 5px 5px 2px 1px rgba(0, 0, 0, 0.5);
  font-family: sans-serif;
  font-size: 11pt;
  padding: 5px;
  color: white;
  outline: 0px;
}


/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-resize-gutter-vertical {
  height: 1px;
  background-color: #333;
  background-clip: content-box;
  padding-top: 2px;
  padding-bottom: 2px;
  cursor: row-resize;
}

.neuroglancer-resize-gutter-horizontal {
  width: 1px;
  background-color: #333;
  background-clip: content-box;
  padding-right: 2px;
  padding-left: 2px;
  cursor: col-resize;
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-layer-side-panel {
  display: flex;
  color: #fff;
  flex-direction: column;
  align-items: stretch;
  flex: 1;
}

.neuroglancer-managed-user-layer-info-panel {
  display: flex;
  flex-direction: column;
  align-items: stretch;
  flex: 1;
}

.neuroglancer-layer-info-panel-container {
  flex: 1;
}


.neuroglancer-layer-side-panel-title {
  display: flex;
  flex-direction: row;
  font-family: sans-serif;
  font-size: 10pt;
  background-color: #393;
  padding: 2px;
}

.neuroglancer-layer-side-panel-name {
  flex: 1;
  background-color: transparent;
  border: 0px;
  color: white;
  outline: 0px;
  /* ensure if name gets too long it doesn't push buttons over */
  width: 0px;
}

.neuroglancer-layer-side-panel-info-panel {
  flex: 1;
  display: flex;
  flex-direction: column;
  font: 10pt sans-serif;
}

.neuroglancer-layer-side-panel-content-container {
  flex: 1;
  display: flex;
  flex-direction: column;
}

.neuroglancer-layer-side-panel-type-measure {
  position: absolute;
  visibility: hidden;
  top: 0px;
  height: auto;
  width: auto;
  white-space: nowrap;
}

.neuroglancer-layer-side-panel-type,
.neuroglancer-layer-side-panel-type-measure {
  padding-left: 3px;
  padding-right: 3px;
  font-weight: bold;
  font-family: sans-serif;
  font-size: 10pt;
  box-sizing: border-box;
  
}

.neuroglancer-layer-side-panel-type {
  -webkit-appearance: none;
  color: white;
  cursor: pointer;
  background-color: #030;
  margin-right: 5px;
  border: 0px;
  outline: 0px;
}

.neuroglancer-layer-side-panel-type option {
  background-color: #000;
}

/**
 * @license
 * Copyright 2020 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-checkbox-icon[data-checked="true"] {
  background-color: rgba(0, 0, 0, 0.5);
}

/**
 * @license
 * Copyright 2020 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-selection-details {
  display: flex;
  flex-direction: column;
  flex: 1;
}

.neuroglancer-selection-details-body {
  height: 0px;
  flex: 1;
  overflow-y: auto;
  flex-basis: 0px;
  min-height: 0px;
}

.neuroglancer-selection-details-titlebar {
  display: flex;
  flex-direction: row;
  align-items: center;
  font-family: sans-serif;
  font-size: 10pt;
  background-color: #03c;
  padding: 2px;
  color: white;
}

.neuroglancer-selection-details-title {
  flex: 1;
}

.neuroglancer-selection-details-position {
  display: flex;
  flex-direction: row;
  align-items: center;
  justify-items: left;
  font-family: monospace;
  font-size: medium;
}

.neuroglancer-selection-details-position-dimension + .neuroglancer-selection-details-position-dimension {
  margin-left: 8px;
}

.neuroglancer-selection-details-position-dimension-name {
  color: #ff6;
}

.neuroglancer-selection-details-position-dimension-coordinate {
  margin-left: 5px;
  user-select: text;
}

.neuroglancer-selection-details-layer {
  margin-top: 4px;
  margin-bottom: 4px;
  border: 1px solid #222;
}

.neuroglancer-selection-details-layer-title {
  cursor: pointer;
  background-color: #222;
  font-family: sans-serif;
  font-size: 10pt;
}

.neuroglancer-selection-details-layer-title:hover {
  background-color: #333;
}

.neuroglancer-selection-details-layer-body {
  padding: 2px;
  display: flex;
  flex-direction: column;
}

.CodeMirror-foldmarker {
  color: blue;
  text-shadow: #b9f 1px 1px 2px, #b9f -1px -1px 2px, #b9f 1px -1px 2px, #b9f -1px 1px 2px;
  font-family: arial;
  line-height: .3;
  cursor: pointer;
}
.CodeMirror-foldgutter {
  width: .7em;
}
.CodeMirror-foldgutter-open,
.CodeMirror-foldgutter-folded {
  cursor: pointer;
}
.CodeMirror-foldgutter-open:after {
  content: "\25BE";
}
.CodeMirror-foldgutter-folded:after {
  content: "\25B8";
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


.neuroglancer-state-editor {
  width: 80%;
}

/**
 * @license
 * Copyright 2019 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-statistics-panel {
  overflow-x: auto;
  overflow-y: auto;
}

.neuroglancer-statistics-panel > table > thead td {
  position: sticky;
  top: 0px;
  zIndex: 2;
  background-color: black;
}

.neuroglancer-statistics-panel > table > tbody tr:hover {
  background-color: #333;
}

/**
 * @license
 * Copyright 2018 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

.neuroglancer-annotation-tool-status-widget {
  color: #3c3;
  font: 10pt sans-serif;
}

/**
 * @license
 * Copyright 2016 Google Inc.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

html, body {
  margin: 0px;
  padding: 0px;
  overflow: hidden;
  width: 100vw;
  height: 100vh;
  background-color: black;
  color: white;
}

body {
  position: relative;
  display: flex;
  flex-direction: column;
}

#neuroglancer-container {
  width: 100vw;
  flex: 1;
}


/*# sourceMappingURL=main.css.map*/